//Calendar implementation for the HTWK Leipzig timetable. Evaluation and display of the individual dates in iCal format.
//Copyright (C) 2024 HTWKalender support@htwkalender.de

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU Affero General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Affero General Public License for more details.

//You should have received a copy of the GNU Affero General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

package main

import (
	"github.com/gofiber/fiber/v3"
	"github.com/gofiber/fiber/v3/middleware/logger"
	"htwkalender/ical/model"
	"htwkalender/ical/service"
	"htwkalender/ical/service/connector/grpc"
	"log/slog"
	"os"
)

// main function for the ical service
// uses rest api to get the data from the data-manager
// exposes rest api endpoints with fiber to serve the data for clients
func main() {

	// get host from env if this is not set use default value 127.0.0.1

	host := os.Getenv("DATA_MANAGER_URL")
	if host == "" {
		host = "127.0.0.1"
	}

	grpcClient := grpc.ConnectGRPCServer(host)

	// Close the grpc connection when the main function ends
	defer grpc.CloseGRPCServer(grpcClient)

	// Log the grpc connection
	// Test the connection to the grpc server
	grpcClient.Connect()
	slog.Info("GRPC connection state", "state", grpcClient.GetState())

	// Initialize a new Fiber app
	webdavRequestMethods := []string{"PROPFIND", "MKCOL", "COPY", "MOVE"}

	fiberApp := fiber.New(fiber.Config{
		CaseSensitive:  true,
		StrictRouting:  true,
		ServerHeader:   "HTWKalender iCal Service",
		AppName:        "HTWKalender iCal Service",
		RequestMethods: append(fiber.DefaultMethods[:], webdavRequestMethods...),
		TrustProxy:     true,
		TrustProxyConfig: fiber.TrustProxyConfig{
			Proxies:  []string{"172.16.0.0/12", "172.18.0.5", "172.18.0.1"},
			Loopback: true,
		},
		ProxyHeader: "X-Forwarded-For",
	})

	var app = model.AppType{
		GrpcClient:     grpcClient,
		Host:           host,
		Fiber:          fiberApp,
		DataManagerURL: "http://" + host + ":8090",
	}

	fiberApp.Use(logger.New(
		logger.Config{
			Format:     "${time} | ${status} | ${latency} | ${method} - ${path} | ${error}\n",
			TimeFormat: "02-01-2006 15:04:05",
		},
	))

	// Add routes to the app instance for the data-manager ical service
	service.AddFeedRoutes(app)

	slog.Info("Starting ical service", "fiberApp", fiberApp.Listen(":8091"))
}
